package br.ufrn.imd.transfer.service;

import br.ufrn.imd.transfer.domain.Account;
import br.ufrn.imd.transfer.domain.Client;
import br.ufrn.imd.transfer.repository.ClientRepository;
import lombok.AllArgsConstructor;
import lombok.extern.java.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Log
@AllArgsConstructor
@Service
public class TransferService {
    @Autowired
    private final ClientRepository clientRepository;

    public Client transfer(Double value, Client fromClient, Account toAccount) {
        fromClient = clientRepository.findById(fromClient.getId())
                .orElseThrow(() -> new RuntimeException("Invalid user!"));

        Account finalToAccount = toAccount;
        Client toClient = clientRepository.findByAccountNumber(toAccount.getNumber())
                .orElseThrow(() -> new RuntimeException(
                        String.format("Customer not found with account number %s", finalToAccount.getNumber())));

        if(fromClient.getId().equals(toClient.getId()))
            throw new RuntimeException("Invalid account");

        Account fromAccount = fromClient.getAccount();
        if(value > fromAccount.getBalance()) throw new RuntimeException("Insufficient balance");

        fromAccount.setBalance(fromAccount.getBalance() - value);
        toAccount = toClient.getAccount();
        toAccount.setBalance(toAccount.getBalance() + value);

        clientRepository.save(toClient);
        fromClient = clientRepository.save(fromClient);
        log.info("Transfer concluded!");
        log.info(String.format("Account balance: R$ %.2f", fromClient.getAccount().getBalance()));

        return fromClient;
    }

}
